<?php
namespace ViceElementor;

/**
 * Class Plugin
 *
 * Main Plugin class
 * @since 1.0.0
 */
class Plugin {
 
	/**
	 * Instance
	 *
	 * @since 1.0.0
	 * @access private
	 * @static
	 *
	 * @var Plugin The single instance of the class.
	 */
	private static $_instance = null;
 
	/**
	 * Instance
	 *
	 * Ensures only one instance of the class is loaded or can be loaded.
	 *
	 * @since 1.2.0
	 * @access public
	 *
	 * @return Plugin An instance of the class.
	 */
	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
					 
		return self::$_instance;
	}
 
	/**
	 * widget_scripts
	 *
	 * Load required plugin core files.
	 *
	 * @since 1.2.0
	 * @access public
	 */
	public function widget_scripts() {

		// COMPONENTS
		// ==================
		$deps = array('jquery', 'masonry');

		// Owl Carousel
		wp_enqueue_script( 'owl-carousel', plugins_url( 'assets/components/owl-carousel/dist/owl.carousel-min.js' , __FILE__ ), $deps,'2.3.2', true);$deps[] = 'owl-carousel';
		// Owl CSS
		wp_enqueue_style( 'owl-carousel', plugins_url( 'assets/components/owl-carousel/dist/assets/owl.carousel.min.css' , __FILE__ ), false, '2.3.2');

		// Main JS
		wp_enqueue_script( 'vice-extend', plugins_url( '/assets/js/extend.js', __FILE__ ), $deps, filemtime( __FILE__ ), true );

		// Main CSS
		wp_enqueue_style( 'vice-elementor', plugins_url( '/assets/css/extend.css', __FILE__ ) );
	}

	/**
	 * enqueue_styles
	 *
	 * Load custom styling.
	 *
	 * @since 1.2.0
	 * @access public
	 */
	public function enqueue_styles() {
		wp_enqueue_style( 'vice-elementor-admin', plugins_url( '/assets/css/admin.css', __FILE__ ) );
		wp_enqueue_style( 'vice-elementor-admin-icons', plugins_url( '/assets/font/vice-for-elementor/styles.css', __FILE__ ) );
		wp_enqueue_style(
			'elementor-editor-dark-mode',
			ELEMENTOR_ASSETS_URL . 'css/editor-dark-mode.css',
			[
				'elementor-editor',
			]
		);
	}

 
	/**
	 * Include Widgets files
	 *
	 * Load widgets files
	 *
	 * @since 1.2.0
	 * @access private
	 */
	private function include_widgets_files() {
		require_once( __DIR__ . '/widgets/_helpers/autocomplete.php' );
		require_once( __DIR__ . '/widgets/_helpers/converter.php' );
		require_once( __DIR__ . '/widgets/_helpers/get-terms-array.php' );
		require_once( __DIR__ . '/widgets/_helpers/query-fields.php' );
		require_once( __DIR__ . '/widgets/_helpers/carousel-fields.php' );
		require_once( __DIR__ . '/widgets/_helpers/slider-fields.php' );
		require_once( __DIR__ . '/widgets/button/button.php' );
		require_once( __DIR__ . '/widgets/caption/caption.php' );
		require_once( __DIR__ . '/widgets/fx-picture/fx-picture.php' );
		require_once( __DIR__ . '/widgets/post-carousel/post-carousel.php' );
		require_once( __DIR__ . '/widgets/post-modernslider/post-modernslider.php' );
		require_once( __DIR__ . '/widgets/moderngallery/moderngallery.php' );
		require_once( __DIR__ . '/widgets/event-list/event-list.php' );
		require_once( __DIR__ . '/widgets/eventsmap/eventsmap.php' );
		require_once( __DIR__ . '/widgets/cf7/cf7.php' );
	}
 
	/**
	 * Register Widgets
	 *
	 * Register new Elementor widgets.
	 *
	 * @since 1.2.0
	 * @access public
	 */
	public function register_widgets() {
		// Register Widgets
		$this->include_widgets_files();
		\Elementor\Plugin::instance()->widgets_manager->register_widget_type( new Widgets\ViceElementorButton() );
		\Elementor\Plugin::instance()->widgets_manager->register_widget_type( new Widgets\ViceElementorCaption() );
		\Elementor\Plugin::instance()->widgets_manager->register_widget_type( new Widgets\ViceElementorFxPicture() );
		\Elementor\Plugin::instance()->widgets_manager->register_widget_type( new Widgets\ViceElementorPostCarousel() );
		\Elementor\Plugin::instance()->widgets_manager->register_widget_type( new Widgets\ViceElementorPostModernSlider() );
		\Elementor\Plugin::instance()->widgets_manager->register_widget_type( new Widgets\ViceElementorModernGallery() );
		\Elementor\Plugin::instance()->widgets_manager->register_widget_type( new Widgets\ViceElementorEventList() );
		\Elementor\Plugin::instance()->widgets_manager->register_widget_type( new Widgets\ViceElementorEventMap() );
		if(defined('WPCF7_VERSION')){
			\Elementor\Plugin::instance()->widgets_manager->register_widget_type( new Widgets\ViceElementorCF7() );
		}
	}

	public function add_elementor_widget_categories($elements_manager ) {
		\Elementor\Plugin::instance()->elements_manager->add_category(
			'Vice',
			[
				'title' => esc_html__( 'Vice ', 'plugin-name' ),
				'icon' => 'fa fa-plug',
			]
		);
	}

	public function order_elementor_widget_categories(\Elementor\Elements_Manager $elements_manager ) {
		$category_prefix = 'aaa-';
		$elements_manager->add_category(
			$category_prefix . 'vice',
			[
				'title' => esc_html__( 'Vice ', 'plugin-name' ),
				'icon' => 'fa fa-plug',
			]
		);
		//hack into the private $categories member, and reorder it so our stuff is at the top
		$reorder_cats = function() use($category_prefix){
			uksort($this->categories, function($keyOne, $keyTwo) use($category_prefix){
				if(substr($keyOne, 0, 4) == $category_prefix){
					return -1;
				}
				if(substr($keyTwo, 0, 4) == $category_prefix){
					return 1;
				}
				return 0;
			});

		};
		$reorder_cats->call($elements_manager);
	}


	/**
	 * ======================================================
	 * Glitch settings parameter
	 * ======================================================
	 */
	// Add glitch Control (Switch) to Section Element in the Editor.
	public function add_elementor_section_background_controls( \Elementor\Element_Section $section ) {
		$section->add_control(
			'vice_section_glitch',
			[
				'label' => __( 'Glitch', 'vice-elementor' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_off' => __( 'Off', 'vice-elementor' ),
				'label_on' => __( 'On', 'vice-elementor' ),
				'default' => 'no',
			]
		);
	}
		
	// Render section backgrou]d parallax
	public function render_elementor_section_glitch_background( \Elementor\Element_Base $element ) {
		$params = $element->get_settings();
		if ( 'yes' === $element->get_settings( 'vice_section_glitch' ) ) {
			$element->add_render_attribute( '_wrapper', 'class', 'vice-elementor-glitchsection' );
		}
	}

	/**
	 * ======================================================
	 * 3D section parameter
	 * ======================================================
	 */
	// Add 3d switch Control to Section Element in the Editor.
	public function add_elementor_section_layout_controls( \Elementor\Element_Section $section ) {
		$section->add_control(
			'vice_section_3d',
			[
				'label' => __( '3D', 'vice-elementor' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_off' => __( 'Off', 'vice-elementor' ),
				'label_on' => __( 'On', 'vice-elementor' ),
				'default' => 'no',
			]
		);
	}
	// Render section attributes
	public function render_elementor_section_layout( \Elementor\Element_Base $element ) {
		$params = $element->get_settings();
		if ( 'yes' === $element->get_settings( 'vice_section_3d' ) ) {
			$element->add_render_attribute( '_wrapper', 'class', 'vice-elementor-3dfx' );
		}
	}

	/**
	 * ======================================================
	 * Plugin class constructor
	 * ======================================================
	 */
	public function __construct() {
		//Add category
		add_action( 'elementor/elements/categories_registered', [ $this, 'add_elementor_widget_categories' ], 0 );
		add_action( 'elementor/elements/categories_registered', [ $this, 'order_elementor_widget_categories' ], 0 );
		// Register widget scripts
		add_action( 'elementor/frontend/after_register_scripts', [ $this, 'widget_scripts' ] );
		// Register widgets
		add_action( 'elementor/widgets/widgets_registered', [ $this, 'register_widgets' ] );
		// custom styling
		add_action('elementor/editor/before_enqueue_scripts', [ $this, 'enqueue_styles' ] );
		// Custom attributes for Elementor
		add_action( 'elementor/element/section/section_background/before_section_end', [ $this,'add_elementor_section_background_controls' ] );
		add_action( 'elementor/frontend/section/before_render', [ $this,'render_elementor_section_glitch_background'] );
		// 3D section
		add_action( 'elementor/element/section/section_layout/before_section_end', [ $this,'add_elementor_section_layout_controls' ] );
		add_action( 'elementor/frontend/section/before_render', [ $this,'render_elementor_section_layout'] );
	}
}
 
// Instantiate Plugin Class
Plugin::instance();
